@@ -1,3 +1,5 @@
+// -*- Mode: C++ -*-
+
 //Author: Dodji Seketeli
 /*
  *This file is part of the Nemiver project
@@ -256,13 +258,12 @@ public:
 
     /// \brief the out of band record we got from GDB.
     ///
-    /// An Out of band record is an aynchronous record that is either
-    /// a set of messages sent by the debugger to tell us about the reason why
-    /// the target has changed state.
+    /// An Out of band record is an aynchronous record that is a set
+    /// of messages sent by the debugger to tell us about how and why
+    /// the target/inferior has changed state.
     class OutOfBandRecord {
     public:
 
-
     private:
         bool m_has_stream_record;
         StreamRecord m_stream_record;
@@ -276,6 +277,8 @@ public:
         long m_thread_id;
         UString m_signal_type;
         UString m_signal_meaning;
+        bool m_has_modified_breakpoint;
+        IDebugger::Breakpoint m_modified_breakpoint;
 
     public:
 
@@ -377,23 +380,49 @@ public:
 
         bool has_signal () const {return m_signal_type != "";}
 
+        /// Getter of the "modified_breakpoint" flag.  This flag is
+        /// true if the underlying debugging engine reports that a
+        /// given breakpoint has been modified.
+        ///
+        /// @return the modified_breakpoint flag.
+        bool has_modified_breakpoint () const
+        {return m_has_modified_breakpoint;}
+
+        /// Getter of the modified breakpoint carried by this out of
+        /// band record.  What this returns is meaningful only if the
+        /// has_modified_breakpoint() member function above returns
+        /// true.
+        IDebugger::Breakpoint& modified_breakpoint ()
+        {return m_modified_breakpoint;}
+
+        /// Setter of the modified breakpoint carried by this out of
+        /// band record.
+        void modified_breakpoint (const IDebugger::Breakpoint& b)
+        {
+            m_modified_breakpoint = b;
+            m_has_modified_breakpoint = true;
+        }
+
         /// @}
 
-        void clear ()
-        {
-            m_has_stream_record = false;
-            m_stream_record.clear ();
-            m_is_stopped = false;
-            m_is_running = false;
-            m_stop_reason = IDebugger::UNDEFINED_REASON;
-            m_has_frame = false;
-            m_thread_selected = false;
-            m_frame.clear ();
-            m_breakpoint_number = 0;
-            m_thread_id = -1;
-            m_signal_type.clear ();
-        }
+	void clear ()
+	{
+	    m_has_stream_record = false;
+	    m_stream_record.clear ();
+	    m_is_stopped = false;
+	    m_is_running = false;
+	    m_stop_reason = IDebugger::UNDEFINED_REASON;
+	    m_has_frame = false;
+	    m_thread_selected = false;
+	    m_frame.clear ();
+	    m_breakpoint_number = 0;
+	    m_thread_id = -1;
+	    m_signal_type.clear ();
+	    m_has_modified_breakpoint = 0;
+	    m_modified_breakpoint.clear();
+	}
     };//end class OutOfBandRecord
+    typedef list<OutOfBandRecord> OutOfBandRecords;
 
     /// \debugger result record
     ///
@@ -418,7 +447,7 @@ public:
 
     private:
         Kind m_kind;
-        map<int, IDebugger::Breakpoint> m_breakpoints;
+        map<string, IDebugger::Breakpoint> m_breakpoints;
         map<UString, UString> m_attrs;
 
         //call stack listed members
@@ -495,7 +524,7 @@ public:
 	// a new member, or removal of last N members), or a change in
 	// one of the children of the variables.
         list<VarChangePtr> m_var_changes;
-        bool m_has_var_changes;	
+        bool m_has_var_changes;
 
 	// Thew new number of children of a dynamic variable.  Set to
 	// -1 by default.
@@ -563,11 +592,11 @@ public:
         Kind kind () const {return m_kind;}
         void kind (Kind a_in) {m_kind = a_in;}
 
-        const map<int, IDebugger::Breakpoint>& breakpoints () const
+        const map<string, IDebugger::Breakpoint>& breakpoints () const
         {
             return m_breakpoints;
         }
-        map<int, IDebugger::Breakpoint>& breakpoints () {return m_breakpoints;}
+        map<string, IDebugger::Breakpoint>& breakpoints () {return m_breakpoints;}
 
         map<UString, UString>& attrs () {return m_attrs;}
         const map<UString, UString>& attrs () const {return m_attrs;}
