@@ -122,7 +122,7 @@ on_stopped_signal (IDebugger::StopReason a_reason,
                    bool /*a_has_frame*/,
                    const IDebugger::Frame &a_frame,
                    int /*a_thread_id*/,
-                   int /*bp num*/,
+                   const string & /*bp num*/,
                    const UString &/*a_cookie*/,
                    IDebuggerSafePtr &a_debugger)
 {
